#include "CircleNode.h"

#include "FontsManager.h"
#include "SceneModel.h"
#include "AbstractMesh.h"
#include "MeshPrimitives.h"
#include "SceneInfo.h"
#include "Circle.h"

CircleNode::CircleNode(const Point& location, double radius)
:    GeometryNode(location),
     mRadius(radius)
{

}

CircleNode::Type CircleNode::getType()
{
    return Type::Circle;
}

double CircleNode::getRadius() const
{
    return mRadius;
}

SceneItem* CircleNode::getSceneItem(std::size_t idx) const
{
    if (idx == 0)
    {
        return mBackgroundItem.get();
    }
    else
    {
        return nullptr;
    }
}

std::size_t CircleNode::getNumSceneItems() const
{
    return 1;
}

void CircleNode::setRadius(double radius)
{
    if (mRadius != radius)
    {
        mRadius = radius;
        mTransformIsDirty = true;
    }
}

void CircleNode::setMinorRadius(double radius)
{
    if (mMinorRadius != radius)
    {
        mMinorRadius = radius;
        mTransformIsDirty = true;
    }
}

void CircleNode::createOrUpdateGeometry(SceneInfo* sceneInfo)
{
    if (sceneInfo->mSupportsGeometryPrimitives)
    {
        auto circle = std::make_unique<Circle>(Point{ 0, 0 }, 0.5);
        circle->setMinorRadius(mMinorRadius);
        mBackgroundItem = std::make_unique<SceneModel>(std::move(circle));
    }
    else
    {
        auto mesh = MeshPrimitives::buildCircleAsTriMesh();
        mBackgroundItem = std::make_unique<SceneModel>(std::move(mesh));
    }
    mBackgroundItem->setName(mName + "_Model");
}

void CircleNode::updateMaterial()
{
    if (!mBackgroundItem)
    {
        return;
    }

    if (mHasFillColor)
    {
        mBackgroundItem->setFillColor(mFillColor);
    }
    if (mHasStrokeColor)
    {
        mBackgroundItem->setOutlineColor(mStrokeColor);
    }
}

void CircleNode::update(SceneInfo* sceneInfo)
{
    if (!mBackgroundItem || mGeometryIsDirty)
    {
        createOrUpdateGeometry(sceneInfo);
        mGeometryIsDirty = false;
    }

    if (mTransformIsDirty)
    {
        mBackgroundItem->updateTransform({mLocation, 2*mRadius, 2*mRadius});
        mTransformIsDirty = false;
    }

    if (mMaterialIsDirty)
    {
        updateMaterial();
        mMaterialIsDirty = false;
    }
}
